﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  // todo: item name/descr...
  [StorableClass]
  public class ParameterConfigurationManipulator : SingleSuccessorOperator, IParameterConfigurationManipulator, IStochasticOperator {
    public override bool CanChangeName {
      get { return false; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ParameterConfigurationTree> ParameterConfigurationTreeParameter {
      get { return (ILookupParameter<ParameterConfigurationTree>)Parameters["ParameterConfigurationTree"]; }
    }

    public ParameterConfigurationManipulator() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
      Parameters.Add(new LookupParameter<ParameterConfigurationTree>("ParameterConfigurationTree", "The parameter configuration which should be manipulated."));
    }
    [StorableConstructor]
    protected ParameterConfigurationManipulator(bool deserializing) : base(deserializing) { }
    protected ParameterConfigurationManipulator(ParameterConfigurationManipulator original, Cloner cloner) : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterConfigurationManipulator(this, cloner);
    }

    public sealed override IOperation Apply() {
      ParameterConfigurationTreeParameter.ActualValue.Mutate(RandomParameter.ActualValue);
      return base.Apply();
    }

  }
}
